// Include DTSource.h if you want to include all the headers.

#include "DTArguments.h"
#include "DTSaveError.h"

#include "DTDataFile.h"
#include "DTSurface3D.h"
#include "DTDoubleArrayOperators.h"

#include <math.h>

DTSurface3D Computation(DTSurface3D input,double a);

int main(int argc,const char *argv[])
{
    DTSetArguments(argc,argv);
    
    DTDataFile inputFile("Input.dtbin",DTFile::ReadOnly);
    // Read in the input variables.
    DTSurface3D input;
    Read(inputFile,"input",input);
    double a = inputFile.ReadNumber("a");
    
    // The computation.
    DTSurface3D computed;
    clock_t t_before = clock();
    computed = Computation(input,a);
    clock_t t_after = clock();
    double exec_time = double(t_after-t_before)/double(CLOCKS_PER_SEC);
    
    // Write the output.
    DTDataFile outputFile("Output.dtbin",DTFile::NewReadWrite);
    
    // Output from computation
    Write(outputFile,"Var",computed);
    outputFile.Save("Surface3D","Seq_Var");
    
    // The execution time.
    outputFile.Save(exec_time,"ExecutionTime");
    outputFile.Save("Real Number","Seq_ExecutionTime");
    
    // The errors.
    DTSaveError(outputFile,"ExecutionErrors");
    outputFile.Save("StringList","Seq_ExecutionErrors");
    
    return 0;
}

DTSurface3D Computation(DTSurface3D input,double a)
{
    DTSurface3D toReturn;

    // input.Points() - DTFloatArray (3xN)
    // input.Connections() - DTIntArray (3xM)
    // input.Normals() - DTFloatArray - (3xN)

    // Do what ever you please
    DTFloatArray points = input.Points();
    int i,howMany = points.n();
    DTMutableFloatArray newPoints(3,howMany);
    for (i=0;i<howMany;i++) {
        newPoints(0,i) = points(0,i);
        newPoints(1,i) = points(1,i);
        newPoints(2,i) = points(2,i)+a*points(0,i);
    }
    
    toReturn = DTSurface3D(input.Connections(),newPoints);

    return toReturn;
}
